Stan Math Library: stan/math/prim/scal/prob/exponential_cdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

exponential_cdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_EXPONENTIAL_CDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_EXPONENTIAL_CDF_LOG_HPP
3
4 #include <boost/random/exponential_distribution.hpp>
5 #include <boost/random/variate_generator.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/value_of.hpp>
12 #include <stan/math/prim/scal/meta/length.hpp>
13 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
14 #include <stan/math/prim/scal/meta/VectorView.hpp>
15 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
16 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
17 #include <stan/math/prim/scal/meta/return_type.hpp>
18 #include <stan/math/prim/scal/fun/constants.hpp>
19 #include <stan/math/prim/scal/meta/include_summand.hpp>
20 #include <cmath>
21
22 namespace stan {
23
24 namespace math {
25
26 template <typename T_y, typename T_inv_scale>
27 typename return_type<T_y, T_inv_scale>::type
28 exponential_cdf_log(const T_y& y, const T_inv_scale& beta) {
29 typedef
30 typename stan::partials_return_type<T_y, T_inv_scale>::type
31 T_partials_return;
32
33 static const char* function("stan::math::exponential_cdf_log");
34
35 using stan::math::check_positive_finite;
36 using stan::math::check_nonnegative;
37 using stan::math::check_not_nan;
38 using boost::math::tools::promote_args;
39 using stan::math::value_of;
40 using std::log;
41 using std::exp;
42
43 T_partials_return cdf_log(0.0);
44 // check if any vectors are zero length
45 if (!(stan::length(y)
46 && stan::length(beta)))
47 return cdf_log;
48
49 check_not_nan(function, "Random variable", y);
50 check_nonnegative(function, "Random variable", y);
51 check_positive_finite(function, "Inverse scale parameter", beta);
52
53 OperandsAndPartials<T_y, T_inv_scale>
54 operands_and_partials(y, beta);
55
56 VectorView<const T_y> y_vec(y);
57 VectorView<const T_inv_scale> beta_vec(beta);
58 size_t N = max_size(y, beta);
59 for (size_t n = 0; n < N; n++) {
60 const T_partials_return beta_dbl = value_of(beta_vec[n]);
61 const T_partials_return y_dbl = value_of(y_vec[n]);
62 T_partials_return one_m_exp = 1.0 - exp(-beta_dbl * y_dbl);
63 // log cdf
64 cdf_log += log(one_m_exp);
65
66 // gradients
67 T_partials_return rep_deriv = -exp(-beta_dbl * y_dbl) / one_m_exp;
68 if (!is_constant_struct<T_y>::value)
69 operands_and_partials.d_x1[n] -= rep_deriv * beta_dbl;
70 if (!is_constant_struct<T_inv_scale>::value)
71 operands_and_partials.d_x2[n] -= rep_deriv * y_dbl;
72 }
73 return operands_and_partials.value(cdf_log);
74 }
75 }
76 }
77
78 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
return_type.hpp
include_summand.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
constants.hpp
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::exponential_cdf_log
return_type< T_y, T_inv_scale >::type exponential_cdf_log(const T_y &y, const T_inv_scale &beta)
Definition: exponential_cdf_log.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.